function Xhbar = calculate_fix_point(X, param)
% Calculates the 'fix point' (in the language of Dynare++) - the fixed
% point of the deterministic version of the nonlinear rule.

Xnew = X.XEXP*0;

max_iter = 10000;
iter = 0;

max_err = 10^(-15);
err = max_err * 2;

block0 = [X.THETA10; X.THETA20];
block1 = [X.THETA11 X.THETA11*0; X.THETA21 X.THETA22];
block2 = [X.THETA23*0; X.THETA23];
n = size(X.THETA10,1);

while ((iter < max_iter) && (err > max_err))
    
    Xold = Xnew;
%     Xnew = X.DELTA + X.A*Xold + 1/2*X.C*kron(Xold,Xold);
    Xnew = block0 + block1*Xold + block2*kron(Xold(1:n),Xold(1:n));
    iter = iter + 1;
    err = norm(Xold - Xnew);
    
end

if (err > max_err)
    warning(sprintf('Calculation of the fix point XHBAR did not converge in %d iterations.', iter));
else
    disp_reg(param,1,sprintf('Calculation of the fix point XHBAR converged in %d iterations.', iter));
end

Xhbar = Xnew;

end