function Hbar = cond_exp_triang(X,H)
% Calculates the conditional expectation of H(X_t,W_t+1) for the triangular
% system in the paper.
%
% X1' = THETA10 + THETA11 X1 + LAMBDA10 W
% X2' = THETA20 + THETA21 X1 + THETA22 X2 + THETA23 kron(X1,X1) +
%       LAMBDA20 W + LAMBDA21 kron(X1,W) + LAMBDA22 kron(W,W)
% H(X,W) = GAMMA0 + GAMMA1 X1 + GAMMA2 X2 + GAMMA3 kron(X1,X1) +
%          PSI0 W + PSI1 kron(X1,W) + PSI2 kron(W,W)

n = size(H.GAMMA1,2);
k = size(H.PSI0,2);

PSI2mat = reshape(H.PSI2,k,k);
PSI2mat = eye(k) - (PSI2mat+PSI2mat');
PSI1mat = reshape(H.PSI1,k,n);

Hbar.GAMMA0 = H.GAMMA0 - 1/2*log(det(PSI2mat)) + 1/2*(H.PSI0/PSI2mat)*H.PSI0';
Hbar.GAMMA1 = H.GAMMA1 + (H.PSI0/PSI2mat)*PSI1mat;
Hbar.GAMMA2 = H.GAMMA2;
Hbar.GAMMA3 = H.GAMMA3 + reshape(1/2*(PSI1mat'/PSI2mat)*PSI1mat,1,n*n);
Hbar.PSI0 = H.PSI0*0;
Hbar.PSI1 = H.PSI1*0;
Hbar.PSI2 = H.PSI2*0;

end