function entropy = construct_entropy(X,M,param)
% Constructs the entropy decomposition described in
% Borovicka, Hansen (2011).

k = size(X.LAMBDA10,2);
n = size(X.LAMBDA10,1);
T = param.model_T;

perlength_scale = 1/param.model_periodlength^0.5;

% =========================================================================
% matrices for elasticities
eGAMMA0 = zeros([1 1 T+1]);
eGAMMA1 = zeros([1 n T+1]);
eGAMMA2 = zeros([1 n T+1]);
eGAMMA3 = zeros([1 n*n T+1]);

eXSTEADY = zeros([1 1 T+1]);
eXEXP = zeros([1 1 T+1]);
eXBAR = zeros([1 1 T+1]);
eXMEAN = zeros([1 1 T+1]);

Miter.GAMMA0 = M.GAMMA0*0;
Miter.GAMMA1 = M.GAMMA1*0;
Miter.GAMMA2 = M.GAMMA2*0;
Miter.GAMMA3 = M.GAMMA3*0;
Miter.PSI0 = M.PSI0*0;
Miter.PSI1 = M.PSI1*0;
Miter.PSI2 = M.PSI2*0;


for t = 1 : T+1
    
    Miter.GAMMA0 = Miter.GAMMA0 + M.GAMMA0;
    Miter.GAMMA1 = Miter.GAMMA1 + M.GAMMA1;
    Miter.GAMMA2 = Miter.GAMMA2 + M.GAMMA2;
    Miter.GAMMA3 = Miter.GAMMA3 + M.GAMMA3;
    Miter.PSI0 = Miter.PSI0 + M.PSI0;
    Miter.PSI1 = Miter.PSI1 + M.PSI1;
    Miter.PSI2 = Miter.PSI2 + M.PSI2;
    
    % ---------------------------------------------------------------------
    % entropy decomposition calculation
    Sigmainv = eye(k)-reshape(2*Miter.PSI2,k,k);
    eGAMMA0cur = perlength_scale^2 * 1/2*((Miter.PSI0/Sigmainv)*Miter.PSI0' - log(det(Sigmainv)) + trace(Sigmainv) - k);
    eGAMMA1cur = perlength_scale^2 * ((Miter.PSI0/Sigmainv)*reshape(Miter.PSI1,k,n));
    eGAMMA3cur = perlength_scale^2 * 1/2*reshape(((reshape(Miter.PSI1,k,n)')/Sigmainv)*reshape(Miter.PSI1,k,n),1,n*n);
    eGAMMA0(1,1,t) = eGAMMA0cur;
    eGAMMA1(1,:,t) = eGAMMA1cur;
    eGAMMA3(1,:,t) = eGAMMA3cur;
    
    % elasticity at the deterministic steady state
    eXSTEADY(:,:,t) = eGAMMA0cur + eGAMMA1cur*X.XHSTEADY(1:n) + eGAMMA3cur*kron(X.XHSTEADY(1:n),X.XHSTEADY(1:n));
    % elasticity at the expansion point of the rule
    eXEXP(:,:,t) = eGAMMA0cur;    
    % elasticity at Xh = XHBAR = fixed point of the deterministic rule
    eXBAR(:,:,t) = eGAMMA0cur + eGAMMA1cur*X.XHBAR(1:n) + eGAMMA3cur*kron(X.XHBAR(1:n),X.XHBAR(1:n));
    % elasticity at Xh = XHMEAN = mean of the stationary
    % distribution, mean calculated by Dynare(++)
    eXMEAN(:,:,t) = eGAMMA0cur + eGAMMA1cur*X.XHMEAN(1:n) + eGAMMA3cur*kron(X.XHMEAN(1:n),X.XHMEAN(1:n));

    % ---------------------------------------------------------------------
    Mbar = cond_exp_triang(X,Miter);
    Miter = shift_1period_triang(X,Miter);
    
end

entropy.GAMMA0 = eGAMMA0;
entropy.GAMMA1 = eGAMMA1;
entropy.GAMMA2 = eGAMMA2;
entropy.GAMMA3 = eGAMMA3;
% entropy.PSI0 = ePSI0;
% entropy.PSI1 = ePSI1;
% entropy.PSI2 = ePSI2;
entropy.XSTEADY = eXSTEADY;
entropy.XEXP = eXEXP;
entropy.XBAR = eXBAR;
entropy.XMEAN = eXMEAN;

entropy.type = param.type;
entropy.cf_name = param.cf_name;
entropy.cf_index = param.cf_index;
entropy.sdf_name = param.sdf_name;
entropy.sdf_index = param.sdf_index;

if (~isempty(param.sdf_name))
    elast.name = sprintf('entropy decomposition - %s', ...
        strrep(cell2mat(param.sdf_name), '_','\_'));
else
    elast.name = sprintf('entropy decomposition - %s', ...
        strrep(cell2mat(param.cf_name), '_','\_'));
end

end
