function elast = construct_shock_elasticities(X,M,param,Eexposure)
% Constructs the elasticities for T periods, taking the triangular
% structure of the system into account.

k = size(X.LAMBDA10,2);
n = size(X.LAMBDA10,1);
T = param.model_T;

perlength_scale = 1/param.model_periodlength^0.5;

% =========================================================================
% matrices for elasticities
eGAMMA0 = zeros([k 1 T+1]);
eGAMMA1 = zeros([k n T+1]);
eGAMMA2 = zeros([k n T+1]);
eGAMMA3 = zeros([k n*n T+1]);
% ePSI0 = zeros([k k T+1]);
% ePSI1 = zeros([k n*k T+1]);
% ePSI2 = zeros([k k*k T+1]);

eXSTEADY = zeros([k 1 T+1]);
eXEXP = zeros([k 1 T+1]);
eXBAR = zeros([k 1 T+1]);
eXMEAN = zeros([k 1 T+1]);

eGAMMA1cur = zeros(k,n);

Miter.GAMMA0 = M.GAMMA0*0;
Miter.GAMMA1 = M.GAMMA1*0;
Miter.GAMMA2 = M.GAMMA2*0;
Miter.GAMMA3 = M.GAMMA3*0;
Miter.PSI0 = M.PSI0*0;
Miter.PSI1 = M.PSI1*0;
Miter.PSI2 = M.PSI2*0;


for t = 1 : T+1
    
    Miter.GAMMA0 = Miter.GAMMA0 + M.GAMMA0;
    Miter.GAMMA1 = Miter.GAMMA1 + M.GAMMA1;
    Miter.GAMMA2 = Miter.GAMMA2 + M.GAMMA2;
    Miter.GAMMA3 = Miter.GAMMA3 + M.GAMMA3;
    Miter.PSI0 = Miter.PSI0 + M.PSI0;
    Miter.PSI1 = Miter.PSI1 + M.PSI1;
    Miter.PSI2 = Miter.PSI2 + M.PSI2;
    
    % ---------------------------------------------------------------------
    % shock elasticity calculation
    if (strcmp(param.type,'price'))
        % price elasticity - annualized

        eGAMMA0cur = Eexposure.GAMMA0(:,:,t) ...
            - perlength_scale * ((eye(k)-reshape(2*Miter.PSI2,k,k)) \ Miter.PSI0');
        eGAMMA1cur(:,1:n) = Eexposure.GAMMA1(:,1:n,t) ...
            - perlength_scale * ((eye(k)-reshape(2*Miter.PSI2,k,k)) \ reshape(Miter.PSI1,k,n));
        eGAMMA0(:,:,t) = eGAMMA0cur;
        eGAMMA1(:,:,t) = eGAMMA1cur;

    else
        % exposure elasticity - annualized
        eGAMMA0cur = perlength_scale * ((eye(k)-reshape(2*Miter.PSI2,k,k)) \ Miter.PSI0');
        eGAMMA1cur(:,1:n) = perlength_scale * ((eye(k)-reshape(2*Miter.PSI2,k,k)) \ reshape(Miter.PSI1,k,n));
        eGAMMA0(:,:,t) = eGAMMA0cur;
        eGAMMA1(:,:,t) = eGAMMA1cur;
    end
    
    % elasticity at the deterministic steady state
    eXSTEADY(:,:,t) = eGAMMA0cur + eGAMMA1cur*X.XHSTEADY(1:n);
    % elasticity at the expansion point of the rule
    eXEXP(:,:,t) = eGAMMA0cur;    
    % elasticity at Xh = XHBAR = fixed point of the deterministic rule
    eXBAR(:,:,t) = eGAMMA0cur + eGAMMA1cur*X.XHBAR(1:n);
    % elasticity at Xh = XHMEAN = mean of the stationary
    % distribution, mean calculated by Dynare(++)
    eXMEAN(:,:,t) = eGAMMA0cur + eGAMMA1cur*X.XHMEAN(1:n);

    % ---------------------------------------------------------------------
    Mbar = cond_exp_triang(X,Miter);
    Miter = shift_1period_triang(X,Mbar);
    
end

elast.GAMMA0 = eGAMMA0;
elast.GAMMA1 = eGAMMA1;
elast.GAMMA2 = eGAMMA2;
elast.GAMMA3 = eGAMMA3;
% elast.PSI0 = ePSI0;
% elast.PSI1 = ePSI1;
% elast.PSI2 = ePSI2;
elast.XSTEADY = eXSTEADY;
elast.XEXP = eXEXP;
elast.XBAR = eXBAR;
elast.XMEAN = eXMEAN;

elast.type = param.type;
elast.cf_name = param.cf_name;
elast.cf_index = param.cf_index;
elast.sdf_name = param.sdf_name;
elast.sdf_index = param.sdf_index;

if (strcmp(param.type,'price'))
    elast.name = sprintf('shock-price elasticity - %s and %s', ...
        strrep(cell2mat(param.sdf_name), '_','\_'), ...
        strrep(cell2mat(param.cf_name), '_','\_'));
elseif (strcmp(param.type,'exposure'))
    elast.name = sprintf('shock-exposure elasticity - %s', ...
        strrep(cell2mat(param.cf_name), '_','\_'));
end

end
