% Load default parameters and overwrite them with user-defined values.
%    Paremeter files are processed in the following order:
%    a) 'default_elast_parameters.m' - stored in the toolbox directory,
%       searched automatically
%    b) 'MODELNAME_elast_parameters.m' - stored in the directory where
%       the model solution file is. For example, the toolbox would
%       search for 'examples/example_RBC_elast_parameters.m' for the model
%       file solution 'example_RBC.mat' in directory 'examples/'.
%       This file does not need to exist.
%    c) 'myparamfile.m' - if it is specified as the second argument
%       when running dynare_elasticity.
%       This file should be stored in the same directory as the model
%       solution file.
%       Example code:
%           elast = dynare_elasticity('example_RBC','myparamfile');

% =========================================================================
% load default parameters
default_elast_parameters;

% search for MODELNAME_elast_parameters.m
if ((~isempty(param.data_filename)) && (exist(sprintf('%s_elast_parameters.m',param.data_filename),'file')))
    param_file_1 = 1;
    eval (sprintf('%s_elast_parameters', param.data_filename));
else
    param_file_1 = 0;
end

% search for user-specific parameter file
if ((~isempty(param.parameter_filename)) && (exist(sprintf('%s.m',param.parameter_filename),'file')))
    param_file_2 = 1;
    eval (sprintf('%s', param.parameter_filename));
else
    param_file_2 = 0;
end

% print initial welcome message
if ((exist('disp_init_message','var')) && (disp_init_message))
    disp_reg(param,0,sprintf('==================================================================='));
    disp_reg(param,0,'Shock-price and shock-exposure elasticity calculation.');
end

% print out everything (waiting for the env_display variables)
disp_reg(param,0,'Default parameter file loaded (default_elast_parameters.m).');
if (param_file_1)
    disp_reg(param,0,sprintf('Parameter file %s_elast_parameters.m loaded.', param.data_filename));
end
if (param_file_2)
    disp_reg(param,0,sprintf('Parameter file %s.m loaded.', param.parameter_filename));
end
if ((~isempty(param.parameter_filename)) && (~exist(sprintf('%s.m',param.parameter_filename),'file')))
    disp_reg(param,0,sprintf('Parameter file %s.m not found.', param.parameter_filename));
end

