function data = plot_elast_plotgraph(param,E)
% Plots an individual graph.

if (isempty(param.plot_shock_indices))
    param.plot_shock_indices = [1:size(E.GAMMA0,1)];
end

model_T = [1:param.model_T+1];

data_content = model_T'*param.model_periodlength/param.plot_periodlength;
data_header = sprintf('# t(%s)',param.plot_periodname);
data_format = '%e';

hold on;

% elasticity at Xh = XHSTEADY (deterministic steady state)
if (param.plot_doxsteady)
    for ii = 1 : length(param.plot_shock_indices)
        i = param.plot_shock_indices(ii);
        plot(model_T*param.model_periodlength/param.plot_periodlength, ...
             param.plot_elastscale*squeeze(E.XSTEADY(i,1,:)), ...
             cell2mat(param.plot_lineorder(mod(ii-1,length(param.plot_lineorder))+1)), ...
             'LineWidth',param.plot_linewidth);
        data_content = [data_content param.plot_elastscale*squeeze(E.XSTEADY(i,1,:))];
        data_header = strcat(data_header,sprintf(' XSTEADY_s%d',i));
        data_format = strcat(data_format,'  %e');
    end
end
% elasticity at Xh = 0 (X = XEXP, expansion point around which the rule is
%                       expressed)
if (param.plot_doxexp)
    for ii = 1 : length(param.plot_shock_indices)
        i = param.plot_shock_indices(ii);
        plot(model_T*param.model_periodlength/param.plot_periodlength, ...
             param.plot_elastscale*squeeze(E.XEXP(i,1,:)), ...
             cell2mat(param.plot_lineorder(mod(ii-1,length(param.plot_lineorder))+1)), ...
             'LineWidth',param.plot_linewidth);
        data_content = [data_content param.plot_elastscale*squeeze(E.XEXP(i,1,:))];
        data_header = strcat(data_header,sprintf(' XEXP_s%d',i));
        data_format = strcat(data_format,'  %e');
    end
end
% elasticity at Xh = XHBAR (fixed point of the deterministic rule)
if (param.plot_doxbar)
    for ii = 1 : length(param.plot_shock_indices)
        i = param.plot_shock_indices(ii);
        plot(model_T*param.model_periodlength/param.plot_periodlength, ...
             param.plot_elastscale*squeeze(E.XBAR(i,1,:)), ...
             cell2mat(param.plot_lineorder(mod(ii-1,length(param.plot_lineorder))+1)), ...
             'LineWidth',param.plot_linewidth);
        data_content = [data_content param.plot_elastscale*squeeze(E.XBAR(i,1,:))];
        data_header = strcat(data_header,sprintf(' XBAR_s%d',i));
        data_format = strcat(data_format,'  %e');
    end
end
% elasticity at Xh = XHMEAN (mean of the distribution, produced by Dynare)
if (param.plot_doxmean)
    for ii = 1 : length(param.plot_shock_indices)
        i = param.plot_shock_indices(ii);
        plot(model_T*param.model_periodlength/param.plot_periodlength, ...
             param.plot_elastscale*squeeze(E.XMEAN(i,1,:)), ...
             cell2mat(param.plot_lineorder(mod(ii-1,length(param.plot_lineorder))+1)), ...
             'LineWidth',param.plot_linewidth);
        data_content = [data_content param.plot_elastscale*squeeze(E.MEAN(i,1,:))];
        data_header = strcat(data_header,sprintf(' XMEAN_s%d',i));
        data_format = strcat(data_format,'  %e');
    end
end

% plot quantiles
if ((param.plot_doquantiles) && (param.simul_calc_shockpriceelast))
    for j = 1 : length(param.simul_quantiles)
        for ii = 1 : length(param.plot_shock_indices)
            i = param.plot_shock_indices(ii);
            plot(model_T*param.model_periodlength/param.plot_periodlength, ...
                 param.plot_elastscale*squeeze(E.simul_quantiles(i,j,:)), ...
                 cell2mat(param.plot_lineorder(mod(ii-1,length(param.plot_lineorder))+1)), ...
                 'LineWidth',1);
        end
    end
end

xlabel(sprintf('maturity (%s)',param.plot_periodname));
xlim([1*param.model_periodlength/param.plot_periodlength param.plot_horizon]);
if (param.plot_graphtitle)
    title(sprintf('%s%s', E.name, param.plot_titlenote),'Interpreter','none');
end
if (~isempty(param.plot_elastunit))
    ylabel(param.plot_elastunit);
end
if (param.plot_graphlegend)
    legend(param.leg_shock_names(param.plot_shock_indices));
end

data.header = data_header;
data.content = data_content;
data.format = data_format;

end