function model = prune_dynamics(model,param)
% Prunes the model solution.

% =========================================================================
% distribute model structure
fn = fieldnames(model);
for i = 1 : length(fn)
    eval(sprintf('%s = model.(cell2mat(fn(i)));',cell2mat(fn(i))));
end

% =========================================================================
disp_reg(param,0,sprintf('Pruning the solution...'));

m = size(X.AX,1); % number of all variables
n = size(X.AX,2); % length of the actual state vector
k = size(X.AW,2); % length of the shock vector

nonstate_vars = setdiff([1:m],state_vars);
XSnew = [ X.XS(state_vars); X.XS(state_vars); X.XS(nonstate_vars) ];
XHMEANnew = [ X.XHMEAN(state_vars); zeros(length(state_vars),1); X.XHMEAN(nonstate_vars) ];
A0new = [ X.A0(state_vars); zeros(n,1); X.A0(nonstate_vars) ];
A1new = [ X.A1(state_vars,1:n) zeros(n,n) X.A1(state_vars,n+1:n+k);
          zeros(n,n) X.A1(state_vars,1:n) X.A1(state_vars,n+1:n+k);
          X.A1(nonstate_vars,1:n) zeros(m-n,n) X.A1(nonstate_vars,n+1:n+k)];
A2part1 = zeros(n,(2*n+k)*(2*n+k));
A2part2 = zeros(m-n,(2*n+k)*(2*n+k));
for i = 1 : n
    a = X.A2(state_vars(i),:);
    mata = mat(a,n+k,n+k);
    mata = 1/2*(mata+mata');
    mataa = [zeros(n,2*n+k); zeros(n+k,n) mata];
    A2part1(i,:) = transpose(mataa(:));
end
for i = 1 : m-n
    a = X.A2(nonstate_vars(i),:);
    mata = mat(a,n+k,n+k);
    mata = 1/2*(mata+mata');
    mataa = [zeros(n,2*n+k); zeros(n+k,n) mata];
    A2part2(i,:) = transpose(mataa(:));
end
A2new = [A2part1; zeros(n,(2*n+k)^2); A2part2];

% X.XSold = X.XS;
% X.A0old = X.A0;
% X.A1old = X.A1;
% X.A2old = X.A2;

X.XS = XSnew;
X.XHMEAN = XHMEANnew;
X.A0 = A0new;
X.A1 = A1new;
X.A2 = A2new;

% rewrite previously saved indices
state_vars_old = state_vars;
state_vars = [1:2*n];
state_vars_firstsecond = [1:n];
state_vars_first = [n+1:2*n];

% state_vars = param.state_vars;

% create a new list of variable names
newlist = repmat(' ',m+n,size(var_names,2)+6);
newlist(1:n,1:size(var_names,2)) = var_names(state_vars_old,:);
newlist(2*n+1:n+m,1:size(var_names,2)) = var_names(nonstate_vars,:);
newlist_tex = repmat(' ',m+n,size(var_names_tex,2)+12);
newlist_tex(1:n,1:size(var_names_tex,2)) = var_names_tex(state_vars_old,:);
newlist_tex(2*n+1:n+m,1:size(var_names_tex,2)) = var_names_tex(nonstate_vars,:);
for i = 1 : n
    st = strcat(strtrim(var_names(state_vars_old(i),:)),'-first');
    newlist(n+i,1:length(st)) = st;
    st = strcat(strtrim(var_names_tex(state_vars_old(i),:)),' first-order');
    newlist_tex(n+i,1:length(st)) = st;
end

model.var_names = newlist;
model.var_names_tex = newlist_tex;
model.state_vars = state_vars;

m = m + n;
n = 2*n;

% now decompose dynAfull AGAIN into X and W components

state_index = [ones(n,1); zeros(k,1)];
shock_index = [zeros(n,1); ones(k,1)];

for ii = 1 : model.expansion_order
    for i = 0 : ii
        varname = '';
        Wkron = 1;
        for j = 1 : i
            Wkron = kron(Wkron,shock_index);
            varname = strcat('W',varname);
        end
        Xkron = 1;
        for j = 1 : ii - i
            Xkron = kron(Xkron,state_index);
            varname = strcat('X',varname);
        end
        XWkron = kron(Xkron,Wkron);
        XWkron = (XWkron == 1);
        % these quantities are scaled by 1 / order!, where order is the
        % current order of expansion
        dynAfull = X.(sprintf('A%d',ii));
        X.(sprintf('A%s',varname)) = dynAfull(:,XWkron);        

    end
end

model.X = X;
    
end