function Hhat = shift_1period_triang(X,H)
% Takes H(X') and the law of motion for X and calculates H(X,W) for the
% triangular system in the paper.
%
% X1' = THETA10 + THETA11 X1 + LAMBDA10 W
% X2' = THETA20 + THETA21 X1 + THETA22 X2 + THETA23 kron(X1,X1) +
%       LAMBDA20 W + LAMBDA21 kron(X1,W) + LAMBDA22 kron(W,W)
% H(X,W) = GAMMA0 + GAMMA1 X1 + GAMMA2 X2 + GAMMA3 kron(X1,X1) +
%          PSI0 W + PSI1 kron(X1,W) + PSI2 kron(W,W)

Hhat.GAMMA0 = H.GAMMA0 + H.GAMMA1*X.THETA10 + H.GAMMA2*X.THETA20 ...
              + H.GAMMA3*X.kTHETA10THETA10;
Hhat.GAMMA1 = H.GAMMA1*X.THETA11 + H.GAMMA2*X.THETA21 ...
               + H.GAMMA3*X.ksumTHETA10THETA11;
Hhat.GAMMA2 = H.GAMMA2*X.THETA22;
Hhat.GAMMA3 = H.GAMMA3*X.kTHETA11THETA11 + H.GAMMA2*X.THETA23;
Hhat.PSI0 = H.GAMMA1*X.LAMBDA10 + H.GAMMA2*X.LAMBDA20 ...
              + H.GAMMA3*X.ksumTHETA10LAMBDA10;
Hhat.PSI1 = H.GAMMA3*X.ksumTHETA11LAMBDA10 + H.GAMMA2*X.LAMBDA21;
Hhat.PSI2 = H.GAMMA3*X.kLAMBDA10LAMBDA10 + H.GAMMA2*X.LAMBDA22;

end