function objects = simulate_distribution(X,param,objects)
% Calculates the quantiles of all objects (incl. shock-price and
% shock-exposure elasticities) under the simulated stationary distribution
% of X.

% =========================================================================
% process all stored objects and calculate the quantiles
% specified in param.simul_quantiles
%
% Shock elasticities are linear functions of the first derivative of the
% state, X1, and so the quadratic dynamics is not needed.

disp_reg(param,0,sprintf('Calculating distributional properties.'));
fnames = fieldnames(objects);

disp_reg(param,1,sprintf('==================================================================='));
n = size(X.AW,1); % number of all state variables
k = size(X.AW,2); % number of shocks
p = param.simul_paths;

Xstore = simulate_paths(X,param);
X1store = Xstore(:,1:n/2);
clear Xstore;

n_store = size(X1store,1);
q_ind = floor(param.simul_quantiles*n_store);

% precalculate quadratic terms - not needed now
% XXstore = (reshape(repmat(Xstore,n,1),n_store,n*n) .* repmat(Xstore,1,n));

for i = 1 : length(fnames)
    % each object can be an array of objects, and moreover can store
    % multidimensional arrays as well
    f = objects.(cell2mat(fnames(i)));
    for ii = 1 : length(f(:))
        if (isfield(f(ii),'GAMMA0'))
            for iii = 1 : size(f(ii).GAMMA0,1)      % shock dimension
                
                fH0 = squeeze(f(ii).GAMMA0(iii,:,:));
                if (size(f(ii).GAMMA0,2) == 1)
                    fH0 = fH0';
                end
                fH1 = squeeze(f(ii).GAMMA1(iii,:,:));
                if (size(f(ii).GAMMA1,2) == 1)
                    fH1 = fH1';
                end
%                 fH3 = squeeze(f(ii).GAMMA3(iii,:,:));
%                 if (size(f(ii).GAMMA3,2) == 1)
%                     fH3 = fH3';
%                 end
                
%                 values = repmat(fH0,n_store,1) ...
%                     + Xstore*fH1 + 1/2*XXstore*fH3;
                values = repmat(fH0,n_store,1) + X1store*fH1;

                valuessorted = sort(values,1,'ascend');

                objects.(cell2mat(fnames(i)))(ii).simul_quantiles(iii,:,:) = valuessorted(q_ind,:);                

                disp_reg(param,1, sprintf('Field %d of %d: %s, element %d of %d, row %d of %d (%f sec).', ...
                    i, length(fnames), cell2mat(fnames(i)), ii, length(f(:)), iii,size(f(ii).GAMMA0,1), toc)); 
                
            end
            
        end
        
    end
    
end

end