function Xstore = simulate_paths(X,param)
% Simulate paths for the state vector X to obtain a stationary
% distribution.

if (param.env_displayverbose)
    disp_reg(param,1,sprintf('Simulating stationary distribution.'));
    disp_reg(param,1,sprintf('%d paths, %d periods dropped, %d periods stored.', ...
        param.simul_paths, param.simul_dropperiods, param.simul_recordperiods));
end

n = size(X.AW,1); % number of all state variables
k = size(X.AW,2); % number of shocks
p = param.simul_paths;

% initial values for the paths
Xh_new = repmat(X.XHBAR',p,1);

t = 0;
stored = 0;
zerooutthreshold = param.simul_zerooutthreshold;
sumreset = 0;

Xstore = zeros(param.simul_recordperiods*param.simul_paths,n);

while (stored < param.simul_recordperiods)

    Xh_path = Xh_new;
    
    W = randn(p,k);
    WW = reshape(repmat(W,k,1),p,k*k) .* repmat(W,1,k); 
    XX = reshape(repmat(Xh_path,n,1),p,n*n) .* repmat(Xh_path,1,n);
    XW = reshape(repmat(Xh_path,k,1),p,n*k) .* repmat(W,1,n);
    
    Xh_new = repmat(X.A0',p,1) ...
        + Xh_path*X.AX' + W*X.AW' + XX*X.AXX' + WW*X.AWW' + 2*XW*X.AXW';
    
    % erase diverging paths
    if (zerooutthreshold > 0)
        ind = (sum(abs(Xh_new),2)>zerooutthreshold*n);
        indcount = sum(ind);
        sumreset = sumreset + indcount;
        if (sum(ind(:)) > 0)
            Xh_new(ind,:) = repmat(X.XHBAR',indcount,1);
        end
        isn = sum(sum(isnan(Xh_new)));
        maxx = max(abs(Xh_new(:)));
    end

    % store data
    if (t > param.simul_dropperiods)
        Xstore(1+stored*p:(stored+1)*p,:) = Xh_new; % + repmat(X.XS',p,1);
        stored = stored + 1;       
    end

    t = t + 1;
end

% reshape to order by simulation paths
for i = 1 : n
    Xstore(:,i) = reshape(transpose(reshape(Xstore(:,i),p,param.simul_recordperiods)), ...
        p*param.simul_recordperiods,1);
end

if (zerooutthreshold > 0)
    disp_reg(param,1,sprintf('  Zeroed out on average %4.1f of %d Xs per period, threshold %e.', ...
             sumreset/(param.simul_dropperiods+param.simul_recordperiods), p*n, zerooutthreshold));
end
tim = toc;
disp_reg(param,1,sprintf('  %d paths for %d periods done (%f seconds).', ...
    param.simul_paths, param.simul_recordperiods, tim));

end

